package oracle.jdbc.driver;

import java.io.IOException;
import java.io.PrintStream;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Statement;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Properties;
import oracle.jdbc.dbaccess.DBAccess;
import oracle.jdbc.dbaccess.DBConversion;
import oracle.jdbc.dbaccess.DBError;

public class OracleConnection implements Connection {
   public static final boolean DEBUG = false;
   private static String dll_string = "dll";
   public String url;
   public String user;
   public String password;
   public String database;
   public boolean auto_commit;
   public DBAccess db_access;
   public DBConversion conversion;
   OracleSql oracle_sql;
   Hashtable statement_table;
   boolean closed;
   int default_batch;
   int default_row_prefetch;
   boolean report_remarks;
   int new_row_prefetch;
   OracleStatement statement_holding_line;
   static final int BINARYSTREAM = 0;
   static final int ASCIISTREAM = 1;
   static final int UNICODESTREAM = 2;
   static final int EOJ_NON = 0;
   static final int EOJ_B_TO_A = 1;
   static final int EOJ_B_TO_U = 2;
   static final int EOJ_A_TO_U = 3;
   static final int EOJ_8_TO_A = 4;
   static final int EOJ_8_TO_U = 5;
   static final int EOJ_U_TO_A = 6;
   static final int ASCII_CHARSET = 0;
   static final int NLS_CHARSET = 1;
   public static final int CHAR_TO_ASCII = 0;
   public static final int CHAR_TO_UNICODE = 1;
   public static final int RAW_TO_ASCII = 2;
   public static final int RAW_TO_UNICODE = 3;
   public static final int UNICODE_TO_CHAR = 4;
   public static final int ASCII_TO_CHAR = 5;
   public static final int NONE = 6;

   public OracleConnection(DBAccess access, String ur, String us, String p, String db, Properties info) throws SQLException {
      this.url = ur;
      this.user = us;
      this.db_access = access;
      this.default_row_prefetch = 10;
      this.default_batch = 1;
      this.new_row_prefetch = 0;
      this.oracle_sql = new OracleSql();
      this.statement_table = new Hashtable(10);
      this.closed = false;

      try {
         this.needLine();
         String dll_str = info.getProperty(dll_string);
         this.conversion = this.db_access.logon(us, p, db, info);
         this.default_row_prefetch = this.db_access.getDefaultPrefetch();
         this.setAutoCommit(true);
      } catch (IOException e) {
         DBError.check_error(e);
      }

   }

   public synchronized Statement createStatement() throws SQLException {
      this.trace("Connection.createStatement");
      if (this.closed) {
         DBError.check_error(-8, "createStatement");
      }

      OracleStatement stmt = new OracleStatement(this, 1, this.default_row_prefetch);
      this.add_statement(stmt);
      return stmt;
   }

   public synchronized PreparedStatement prepareStatement(String sql) throws SQLException {
      this.trace("Connection.prepareStatement");
      if (this.closed) {
         DBError.check_error(-8, "prepareStatement");
      }

      OraclePreparedStatement pstmt = new OraclePreparedStatement(this, sql, this.default_batch, this.default_row_prefetch);
      this.add_statement(pstmt);
      return pstmt;
   }

   public synchronized CallableStatement prepareCall(String sql) throws SQLException {
      this.trace("Connection.prepareCall");
      if (this.closed) {
         DBError.check_error(-8, "prepareCall");
      }

      OracleCallableStatement cstmt = new OracleCallableStatement(this, sql, 1, this.default_row_prefetch);
      this.add_statement(cstmt);
      return cstmt;
   }

   public String nativeSQL(String sql) throws SQLException {
      this.trace("Connection.nativeSQL");
      this.trace(sql);
      String osql = this.oracle_sql.parse(sql);
      this.trace(osql);
      return osql;
   }

   public synchronized void setAutoCommit(boolean autoCommit) throws SQLException {
      this.trace("Connection.setAutoCommit");
      if (this.closed) {
         DBError.check_error(-8, "setAutoCommit");
      }

      try {
         this.needLine();
         this.db_access.setAutoCommit(autoCommit);
      } catch (IOException e) {
         DBError.check_error(e);
      }

      this.auto_commit = autoCommit;
   }

   public boolean getAutoCommit() throws SQLException {
      this.trace("Connection.getAutoCommit");
      return this.auto_commit;
   }

   public synchronized void commit() throws SQLException {
      this.trace("Connection.commit");
      if (this.closed) {
         DBError.check_error(-8, "commit");
      }

      Enumeration e = this.statement_table.elements();

      while(e.hasMoreElements()) {
         OracleStatement s = (OracleStatement)e.nextElement();
         s.sendBatch();
      }

      try {
         this.needLine();
         this.db_access.commit();
      } catch (IOException ex) {
         DBError.check_error(ex);
      }

   }

   public synchronized void rollback() throws SQLException {
      this.trace("Connection.rollback");
      if (this.closed) {
         DBError.check_error(-8, "commit");
      }

      try {
         this.needLine();
         this.db_access.rollback();
      } catch (IOException e) {
         DBError.check_error(e);
      }

   }

   public synchronized void finalize() throws SQLException {
      this.trace("Connection.finalize");
      this.close();
   }

   public synchronized void close() throws SQLException {
      this.trace("Connection.close");
      if (!this.closed) {
         this.closed = true;
         this.close_statements();

         try {
            this.needLine();
            this.db_access.logoff();
         } catch (IOException e) {
            DBError.check_error(e);
         }

      }
   }

   public boolean isClosed() throws SQLException {
      this.trace("Connection.isClosed");
      return this.closed;
   }

   public synchronized DatabaseMetaData getMetaData() throws SQLException {
      this.trace("Connection.getMetaData");
      if (this.closed) {
         DBError.check_error(-8, "getMetaData");
      }

      return new OracleDatabaseMetaData(this);
   }

   public void setReadOnly(boolean readOnly) throws SQLException {
      this.trace("Connection.setReadOnly");
      if (readOnly) {
         DBError.check_error(-29, "setReadOnly");
      }

   }

   public boolean isReadOnly() throws SQLException {
      this.trace("Connection.isReadOnly");
      return false;
   }

   public void setCatalog(String catalog) throws SQLException {
      this.trace("Connection.setCatalog");
   }

   public String getCatalog() throws SQLException {
      this.trace("Connection.getCatalog");
      return null;
   }

   public void setTransactionIsolation(int level) throws SQLException {
      this.trace("Connection.setTransactionIsolation");
      if (level != 1) {
         DBError.check_error(-30, "setTransactionIsolation");
      }

   }

   public int getTransactionIsolation() throws SQLException {
      this.trace("Connection.getTransactionIsolation");
      return 1;
   }

   public void setAutoClose(boolean autoClose) throws SQLException {
      this.trace("Connection.setAutoClose");
      if (!autoClose) {
         DBError.check_error(-31, "setAutoClose");
      }

   }

   public boolean getAutoClose() throws SQLException {
      this.trace("Connection.getAutoClose");
      return true;
   }

   public SQLWarning getWarnings() throws SQLException {
      this.trace("Connection.getWarnings");
      return null;
   }

   public void clearWarnings() throws SQLException {
      this.trace("Connection.clearWarnings");
   }

   public synchronized void trace(String s) {
      PrintStream outStream = DriverManager.getLogStream();
      if (outStream != null) {
         outStream.println(s);
      }

   }

   public synchronized void setDefaultRowPrefetch(int value) throws SQLException {
      if (value <= 0) {
         DBError.check_error(-20, "setRowPrefetch");
      }

      this.default_row_prefetch = value;
   }

   public int getDefaultRowPrefetch() {
      return this.default_row_prefetch;
   }

   public synchronized void setDefaultExecuteBatch(int batch) throws SQLException {
      if (batch <= 0) {
         DBError.check_error(-42, "setDefaultExecuteBatch");
      }

      this.default_batch = batch;
   }

   public int getDefaultExecuteBatch() {
      return this.default_batch;
   }

   public void setRemarksReporting(boolean report_remarks) {
      this.report_remarks = report_remarks;
   }

   public boolean getRemarksReporting() {
      return this.report_remarks;
   }

   synchronized void add_statement(Statement s) {
      this.statement_table.put(s, s);
   }

   synchronized void remove_statement(Statement s) {
      this.statement_table.remove(s);
   }

   private synchronized void close_statements() throws SQLException {
      Enumeration e = this.statement_table.elements();

      while(e.hasMoreElements()) {
         Statement s = (Statement)e.nextElement();
         s.close();
      }

   }

   public synchronized void needLine() throws SQLException {
      if (this.statement_holding_line != null) {
         this.statement_holding_line.freeLine();
      }

   }

   public synchronized void holdLine(OracleStatement stmt) {
      this.statement_holding_line = stmt;
   }

   public synchronized void releaseLine() {
      this.statement_holding_line = null;
   }
}
