package oracle.jdbc.driver;

import java.sql.Connection;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.DriverPropertyInfo;
import java.sql.SQLException;
import java.util.Hashtable;
import java.util.Properties;
import oracle.jdbc.dbaccess.DBAccess;
import oracle.jdbc.dbaccess.DBError;

public class OracleDriver implements Driver {
   public static final char slash_character = '/';
   public static final char at_sign_character = '@';
   private static final String oracle_string = "oracle";
   private static final String user_string = "user";
   private static final String password_string = "password";
   private static final String database_string = "database";
   private static final String server_string = "server";
   private static final String access_string = "access";
   private static final String protocol_string = "protocol";
   public static final String dll_string = "dll";
   private static final String prefetch_string = "prefetch";
   private static final String row_prefetch_string = "rowPrefetch";
   private static final String default_row_prefetch_string = "defaultRowPrefetch";
   private static final String batch_string = "batch";
   private static final String execute_batch_string = "executeBatch";
   private static final String default_execute_batch_string = "defaultExecuteBatch";
   private static final String remarks_string = "remarks";
   private static final String report_remarks_string = "remarksReporting";
   private static Properties protocols = new Properties();

   public synchronized Connection connect(String url, Properties info) throws SQLException {
      if (!this.acceptsURL(url)) {
         return null;
      } else {
         Hashtable url_properties = this.parse_url(url);
         String user = info.getProperty("user");
         String password = info.getProperty("password");
         String database = info.getProperty("database");
         if (database == null) {
            database = info.getProperty("server");
         }

         if (user == null) {
            user = (String)url_properties.get("user");
         }

         if (password == null) {
            password = (String)url_properties.get("password");
         }

         if (database == null) {
            database = (String)url_properties.get("database");
         }

         String protocol = (String)url_properties.get("protocol");
         if (protocol == null) {
            DBError.check_error(-40, "OracleDriver.parse_url");
         }

         String access = (String)protocols.get(protocol);
         if (access == null) {
            DBError.check_error(-40, "OracleDriver.parse_url");
         }

         String dll_str = info.getProperty("dll");
         if (dll_str == null) {
            if (protocol.equals("oci7")) {
               ((Hashtable)info).put("dll", "oci733jdbc");
            } else if (protocol.equals("oci8")) {
               ((Hashtable)info).put("dll", "oci803jdbc");
            }
         }

         String prefetch = info.getProperty("prefetch");
         if (prefetch == null) {
            prefetch = info.getProperty("rowPrefetch");
         }

         if (prefetch == null) {
            prefetch = info.getProperty("defaultRowPrefetch");
         }

         if (prefetch != null && Integer.parseInt(prefetch) <= 0) {
            prefetch = null;
         }

         String batch = info.getProperty("batch");
         if (batch == null) {
            batch = info.getProperty("executeBatch");
         }

         if (batch == null) {
            batch = info.getProperty("defaultExecuteBatch");
         }

         if (batch != null && Integer.parseInt(batch) <= 0) {
            batch = null;
         }

         String remarks = info.getProperty("remarks");
         if (remarks == null) {
            remarks = info.getProperty("remarksReporting");
         }

         DBAccess accessor = null;

         try {
            accessor = (DBAccess)Class.forName(access).newInstance();
         } catch (Exception var15) {
            return null;
         }

         OracleConnection connection = new OracleConnection(accessor, url, user, password, database, info);
         if (prefetch != null) {
            connection.setDefaultRowPrefetch(Integer.parseInt(prefetch));
         }

         if (batch != null) {
            connection.setDefaultExecuteBatch(Integer.parseInt(batch));
         }

         if (remarks != null) {
            connection.setRemarksReporting(remarks.equalsIgnoreCase("true"));
         }

         return connection;
      }
   }

   public boolean acceptsURL(String url) {
      int first_colon = url.indexOf(58);
      if (first_colon == -1) {
         return false;
      } else {
         int second_colon = url.indexOf(58, first_colon + 1);
         return second_colon == -1 ? false : url.regionMatches(true, first_colon + 1, "oracle", 0, second_colon - (first_colon + 1));
      }
   }

   public synchronized DriverPropertyInfo[] getPropertyInfo(String url, Properties info) throws SQLException {
      return new DriverPropertyInfo[0];
   }

   public int getMajorVersion() {
      return 1;
   }

   public int getMinorVersion() {
      return 0;
   }

   public boolean jdbcCompliant() {
      return true;
   }

   Hashtable parse_url(String url) throws SQLException {
      Hashtable result = new Hashtable(5);
      int sub_sub_index = url.indexOf(58, url.indexOf(58) + 1) + 1;
      int end = url.length();
      if (sub_sub_index == end) {
         return result;
      } else {
         int next_colon_index = url.indexOf(58, sub_sub_index);
         if (next_colon_index == -1) {
            return result;
         } else {
            result.put("protocol", url.substring(sub_sub_index, next_colon_index));
            int user = next_colon_index + 1;
            int slash = url.indexOf(47, user);
            int at_sign = url.indexOf(64, user);
            if (at_sign == -1) {
               at_sign = end;
            }

            if (slash == -1) {
               slash = at_sign;
            }

            if (user < slash) {
               result.put("user", url.substring(user, slash));
            }

            if (slash < at_sign) {
               result.put("password", url.substring(slash + 1, at_sign));
            }

            if (at_sign < end) {
               result.put("database", url.substring(at_sign + 1));
            }

            return result;
         }
      }
   }

   String url_protocol(String url, int start) {
      int end = url.indexOf(58, start);
      if (end == -1) {
         return null;
      } else {
         url = url.substring(start, end);
         return protocols.getProperty(url);
      }
   }

   static {
      protocols.put("thin", "oracle.jdbc.ttc7.TTC7Protocol");
      protocols.put("dnldthin", "oracle.jdbc.dnldttc7.TTC7Protocol");
      protocols.put("oci7", "oracle.jdbc.oci7.OCIDBAccess");
      protocols.put("oci8", "oracle.jdbc.oci7.OCIDBAccess");
      protocols.put("3Tthin", "oracle.jdbc.corba.client.Access");
      protocols.put("dnld3Tthin", "oracle.jdbc.dnldcorba.client.Access");

      try {
         DriverManager.registerDriver(new OracleDriver());
      } catch (RuntimeException var0) {
      } catch (SQLException var1) {
      }

   }
}
