package oracle.jdbc.driver;

import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Time;
import java.sql.Timestamp;
import oracle.jdbc.dbaccess.DBAccess;
import oracle.jdbc.dbaccess.DBDataSet;
import oracle.jdbc.dbaccess.DBError;
import oracle.jdbc.dbaccess.DBItem;
import oracle.jdbc.dbaccess.DBType;

public class OraclePreparedStatement extends OracleStatement implements PreparedStatement {
   static final boolean DEBUG = false;
   String sql_query;
   boolean[] last_binds;

   public OraclePreparedStatement(OracleConnection c, String s, int batch_value, int row_prefetch_value) throws SQLException {
      super(c, batch_value, row_prefetch_value);
      super.need_to_parse = true;
      ((OracleStatement)this).prepare_for_new_result();
      this.sql_query = s;
      this.last_binds = new boolean[0];
   }

   public synchronized ResultSet executeQuery() throws SQLException {
      super.connection.trace("PreparedStatement.executeQuery");
      ((OracleStatement)this).prepare_for_new_result();
      ((OracleStatement)this).doExecuteWithTimeout(this.sql_query);
      super.current_result_set = new OracleResultSet(super.connection, this);
      return super.current_result_set;
   }

   void end_of_result_set() throws SQLException {
      ((OracleStatement)this).prepare_for_new_result();
   }

   void clear_last_binds() {
      this.last_binds = new boolean[0];
   }

   public synchronized int executeUpdate() throws SQLException {
      super.connection.trace("PreparedStatement.executeUpdate");
      ((OracleStatement)this).prepare_for_new_result();
      ((OracleStatement)this).doExecuteWithTimeout(this.sql_query);
      this.clear_last_binds();
      return super.valid_rows;
   }

   public synchronized boolean execute() throws SQLException {
      super.connection.trace("PreparedStatement.execute");
      ((OracleStatement)this).prepare_for_new_result();
      ((OracleStatement)this).doExecuteWithTimeout(this.sql_query);
      this.clear_last_binds();
      return super.sql_kind == 0;
   }

   public synchronized int sendBatch() throws SQLException {
      if (super.current_rank > 0) {
         int actual_batch = super.batch;
         int last_current_rank = super.current_rank;

         try {
            --super.current_rank;
            super.batch = super.current_rank;
            ((OracleStatement)this).doExecuteWithTimeout(this.sql_query);
         } finally {
            super.batch = actual_batch;
         }

         for(int index = 0; index < this.last_binds.length; ++index) {
            if (this.last_binds[index]) {
               new DBItem(super.binds_in.data[index].getItem(last_current_rank));
               super.binds_in.setItem(super.binds_in.data[index].getItem(last_current_rank), index, 0);
            }
         }
      }

      return super.valid_rows;
   }

   public synchronized void setExecuteBatch(int new_batch) throws SQLException {
      if (new_batch <= 0) {
         DBError.check_error(-42, "setExecuteBatch");
      }

      this.sendBatch();
      if (new_batch > super.batch) {
         if (super.binds_in.data != null) {
            DBDataSet new_binds_in = new DBDataSet(super.binds_in.types.length, new_batch);

            for(int index = 0; index < super.binds_in.types.length; ++index) {
               if (this.last_binds[index]) {
                  new_binds_in.setType(index, super.binds_in.types[index]);
                  new_binds_in.setItem(super.binds_in.data[index].getItem(0), index, 0);
               }
            }

            super.binds_in = new_binds_in;
         } else {
            super.binds_in = new DBDataSet(new_batch);
         }
      }

      super.batch = new_batch;
   }

   public int getExecuteBatch() {
      return super.batch;
   }

   void checkBindTypes(boolean in, int index, int type) throws SQLException {
      if (index < 0) {
         DBError.check_error(-3, "checkBindTypes");
      }

      DBDataSet dbset1;
      DBDataSet dbset2;
      if (in) {
         dbset1 = super.binds_in;
         dbset2 = super.binds_out;
      } else {
         dbset1 = super.binds_out;
         dbset2 = super.binds_in;
      }

      if (dbset1.types != null && dbset1.types.length > index) {
         DBType bind_type = dbset1.getType(index);
         if (bind_type != null && bind_type.type != type) {
            this.sendBatch();
            super.need_to_parse = true;
            ((OracleStatement)this).clearDefines();
         }
      }

      if (dbset2.types != null && dbset2.types.length > index) {
         DBType bind_type = dbset2.getType(index);
         if (bind_type != null && bind_type.type != type && (bind_type.type != 96 || type != 1) && (bind_type.type != 1 || type != 96)) {
            DBError.check_error(-12, "checkBindTypes");
         }
      }

   }

   public synchronized void setItem(int index, int type, byte[] array) throws SQLException {
      this.checkBindTypes(true, index, type);
      if (this.last_binds.length < index + 1) {
         boolean[] last_binds_copy = this.last_binds;
         this.last_binds = new boolean[index + 1];

         for(int i = 0; i < last_binds_copy.length; ++i) {
            this.last_binds[i] = last_binds_copy[i];
         }

         this.last_binds[index] = true;
      }

      DBType dbtype;
      if (type == 102) {
         int cursor_size = super.connection.db_access.getRefCursorBytesSize();
         dbtype = new DBType(type, cursor_size);
      } else {
         dbtype = new DBType(type);
      }

      DBItem item = new DBItem(dbtype.max_length);
      if (array == null) {
         item.setNull();
      } else {
         item.setArrayData(false, array);
      }

      super.binds_in.needArray(index, dbtype);
      super.binds_in.setItem(item, index, super.current_rank);
   }

   public synchronized void setStreamItem(int index, int type, InputStream istream, int size) throws SQLException {
      if (size < 0) {
         DBError.check_error(-43, "setStreamItem");
      }

      this.checkBindTypes(true, index, type);
      if (this.last_binds.length < index + 1) {
         boolean[] last_binds_copy = this.last_binds;
         this.last_binds = new boolean[index + 1];

         for(int i = 0; index < last_binds_copy.length; ++i) {
            this.last_binds[i] = last_binds_copy[i];
         }

         this.last_binds[index] = true;
      }

      DBType dbtype = new DBType(type, size);
      DBItem item = new DBItem(size);
      item.setStreamData(false, size, istream);
      super.binds_in.needArray(index, dbtype);
      super.binds_in.setItem(item, index, super.current_rank);
   }

   public synchronized void setNull(int parameterIndex, int sqlType) throws SQLException {
      super.connection.trace("PreparedStatement.setNull");
      int index = parameterIndex - 1;
      int internal_type = ((OracleStatement)this).get_internal_type(sqlType);
      this.checkBindTypes(true, index, internal_type);
      DBType dbtype;
      if (internal_type == 102) {
         int cursor_size = super.connection.db_access.getRefCursorBytesSize();
         dbtype = new DBType(internal_type, cursor_size);
      } else {
         dbtype = new DBType(internal_type);
      }

      DBItem item = new DBItem(dbtype.max_length);
      item.setNull();
      super.binds_in.needArray(index, dbtype);
      super.binds_in.setItem(item, index, super.current_rank);
   }

   public synchronized void setBoolean(int parameterIndex, boolean x) throws SQLException {
      int index = parameterIndex - 1;
      byte[] array = super.connection.conversion.BooleanToNumberBytes(x);
      this.setItem(index, 6, array);
   }

   public synchronized void setByte(int parameterIndex, byte x) throws SQLException {
      super.connection.trace("PreparedStatement.setByte");
      int index = parameterIndex - 1;
      byte[] array = super.connection.conversion.ByteToNumberBytes(x);
      this.setItem(index, 6, array);
   }

   public synchronized void setShort(int parameterIndex, short x) throws SQLException {
      super.connection.trace("PreparedStatement.setShort");
      int index = parameterIndex - 1;
      byte[] array = super.connection.conversion.ShortToNumberBytes(x);
      this.setItem(index, 6, array);
   }

   public synchronized void setInt(int parameterIndex, int x) throws SQLException {
      super.connection.trace("PreparedStatement.setInt");
      int index = parameterIndex - 1;
      byte[] array = super.connection.conversion.IntToNumberBytes(x);
      this.setItem(index, 6, array);
   }

   public synchronized void setLong(int parameterIndex, long x) throws SQLException {
      super.connection.trace("PreparedStatement.setLong");
      int index = parameterIndex - 1;
      byte[] array = super.connection.conversion.LongToNumberBytes(x);
      this.setItem(index, 6, array);
   }

   public synchronized void setFloat(int parameterIndex, float x) throws SQLException {
      super.connection.trace("PreparedStatement.setFloat");
      int index = parameterIndex - 1;
      byte[] array = super.connection.conversion.FloatToNumberBytes(x);
      this.setItem(index, 6, array);
   }

   public synchronized void setDouble(int parameterIndex, double x) throws SQLException {
      super.connection.trace("PreparedStatement.setDouble");
      int index = parameterIndex - 1;
      byte[] array = super.connection.conversion.DoubleToNumberBytes(x);
      this.setItem(index, 6, array);
   }

   public synchronized void setBigDecimal(int parameterIndex, BigDecimal x) throws SQLException {
      super.connection.trace("PreparedStatement.setBigDecimal");
      int index = parameterIndex - 1;
      byte[] array = super.connection.conversion.BigDecimalToNumberBytes(x);
      this.setItem(index, 6, array);
   }

   public synchronized void setString(int parameterIndex, String x) throws SQLException {
      super.connection.trace("PreparedStatement.setString");
      int index = parameterIndex - 1;
      byte[] array = super.connection.conversion.StringToCharBytes(x);
      this.setItem(index, 1, array);
   }

   public synchronized void setCursor(int parameterIndex, ResultSet rs) throws SQLException {
      super.connection.trace("PreparedStatement.setCursor");
      int index = parameterIndex - 1;
      DBAccess db = super.connection.db_access;
      byte[] array = null;

      try {
         array = db.DBStatementToRefCursorBytes(((OracleResultSet)rs).statement.dbstmt);
      } catch (IOException e) {
         DBError.check_error(e);
      }

      this.setItem(index, 102, array);
   }

   public synchronized void setRowid(int parameterIndex, OracleRowid oracle_rowid) throws SQLException {
      this.setString(parameterIndex, oracle_rowid.getRowid());
   }

   public synchronized void setBlob(int parameterIndex, OracleBlob lob) throws SQLException {
      this.setItem(parameterIndex - 1, 113, lob.getBytes());
   }

   public synchronized void setClob(int parameterIndex, OracleClob lob) throws SQLException {
      this.setItem(parameterIndex - 1, 112, lob.getBytes());
   }

   public synchronized void setBfile(int parameterIndex, OracleBfile file) throws SQLException {
      this.setItem(parameterIndex - 1, 114, file.getBytes());
   }

   public synchronized void setCfile(int parameterIndex, OracleCfile file) throws SQLException {
      this.setItem(parameterIndex - 1, 115, file.getBytes());
   }

   public synchronized void setBytes(int parameterIndex, byte[] x) throws SQLException {
      super.connection.trace("PreparedStatement.setBytes");
      int index = parameterIndex - 1;
      this.setItem(index, 23, x);
   }

   public synchronized void setDate(int parameterIndex, Date x) throws SQLException {
      super.connection.trace("PreparedStatement.setDate");
      int index = parameterIndex - 1;
      byte[] array = super.connection.conversion.DateToDateBytes(x);
      this.setItem(index, 12, array);
   }

   public synchronized void setTime(int parameterIndex, Time x) throws SQLException {
      super.connection.trace("PreparedStatement.setTime");
      int index = parameterIndex - 1;
      byte[] array = super.connection.conversion.TimeToDateBytes(x);
      this.setItem(index, 12, array);
   }

   public synchronized void setTimestamp(int parameterIndex, Timestamp x) throws SQLException {
      super.connection.trace("PreparedStatement.setTimestamp");
      int index = parameterIndex - 1;
      byte[] array = super.connection.conversion.TimestampToDateBytes(x);
      this.setItem(index, 12, array);
   }

   public synchronized void setAsciiStream(int parameterIndex, InputStream istream, int length) throws SQLException {
      super.connection.trace("PreparedStatement.setBinaryStream");
      int index = parameterIndex - 1;
      this.setExecuteBatch(1);
      InputStream converted_stream = super.connection.conversion.ConvertStream(istream, 5);
      this.setStreamItem(index, 8, converted_stream, length);
   }

   public synchronized void setBinaryStream(int parameterIndex, InputStream istream, int length) throws SQLException {
      super.connection.trace("PreparedStatement.setBinaryStream");
      int index = parameterIndex - 1;
      this.setExecuteBatch(1);
      this.setStreamItem(index, 24, istream, length);
   }

   public synchronized void setUnicodeStream(int parameterIndex, InputStream istream, int length) throws SQLException {
      super.connection.trace("PreparedStatement.setUnicodeStream");
      int index = parameterIndex - 1;
      this.setExecuteBatch(1);
      InputStream converted_stream = super.connection.conversion.ConvertStream(istream, 4);
      this.setStreamItem(index, 8, converted_stream, length);
   }

   public synchronized void setObject(int parameterIndex, Object x, int targetSqlType, int scale) throws SQLException {
      super.connection.trace("PreparedStatement.setObject");
      switch (targetSqlType) {
         case -14:
            this.setCfile(parameterIndex, (OracleCfile)x);
            break;
         case -13:
            this.setBfile(parameterIndex, (OracleBfile)x);
            break;
         case -12:
            this.setClob(parameterIndex, (OracleClob)x);
            break;
         case -11:
            this.setBlob(parameterIndex, (OracleBlob)x);
            break;
         case -10:
            this.setCursor(parameterIndex, (OracleResultSet)x);
            break;
         case -8:
            this.setRowid(parameterIndex, (OracleRowid)x);
            break;
         case -7:
            this.setBoolean(parameterIndex, (Boolean)x);
            break;
         case -6:
            this.setByte(parameterIndex, (byte)(Integer)x);
            break;
         case -5:
            this.setLong(parameterIndex, (Long)x);
            break;
         case -4:
            this.setBytes(parameterIndex, (byte[])x);
            break;
         case -3:
            this.setBytes(parameterIndex, (byte[])x);
            break;
         case -2:
            this.setBytes(parameterIndex, (byte[])x);
            break;
         case -1:
            this.setString(parameterIndex, (String)x);
            break;
         case 1:
            this.setString(parameterIndex, (String)x);
            break;
         case 2:
            this.setBigDecimal(parameterIndex, (BigDecimal)x);
            break;
         case 3:
            this.setBigDecimal(parameterIndex, (BigDecimal)x);
            break;
         case 4:
            this.setInt(parameterIndex, (Integer)x);
            break;
         case 5:
            this.setShort(parameterIndex, (short)(Integer)x);
            break;
         case 6:
            this.setFloat(parameterIndex, (Float)x);
            break;
         case 7:
            this.setFloat(parameterIndex, (Float)x);
            break;
         case 8:
            this.setDouble(parameterIndex, (Double)x);
            break;
         case 12:
            this.setString(parameterIndex, (String)x);
            break;
         case 91:
            this.setDate(parameterIndex, (Date)x);
            break;
         case 92:
            this.setTime(parameterIndex, (Time)x);
            break;
         case 93:
            this.setTimestamp(parameterIndex, (Timestamp)x);
            break;
         default:
            DBError.check_error(-4, "setObject");
      }

   }

   public void setObject(int parameterIndex, Object x, int targetSqlType) throws SQLException {
      super.connection.trace("PreparedStatement.setObject");
      this.setObject(parameterIndex, x, targetSqlType, 0);
   }

   public synchronized void setObject(int parameterIndex, Object x) throws SQLException {
      super.connection.trace("PreparedStatement.setObject");
      int sqlType = this.sqlTypeForObject(x);
      this.setObject(parameterIndex, x, sqlType, 0);
   }

   int sqlTypeForObject(Object x) {
      if (x == null) {
         return 0;
      } else {
         try {
            if ((String)x != null) {
               return 12;
            }
         } catch (Exception var12) {
         }

         try {
            if ((BigDecimal)x != null) {
               return 2;
            }
         } catch (Exception var11) {
         }

         try {
            if ((Boolean)x != null) {
               return -7;
            }
         } catch (Exception var10) {
         }

         try {
            if ((Integer)x != null) {
               return 4;
            }
         } catch (Exception var9) {
         }

         try {
            if ((Long)x != null) {
               return -5;
            }
         } catch (Exception var8) {
         }

         try {
            if ((Float)x != null) {
               return 6;
            }
         } catch (Exception var7) {
         }

         try {
            if ((Double)x != null) {
               return 8;
            }
         } catch (Exception var6) {
         }

         try {
            if ((byte[])x != null) {
               return -3;
            }
         } catch (Exception var5) {
         }

         try {
            if ((Date)x != null) {
               return 91;
            }
         } catch (Exception var4) {
         }

         try {
            if ((Time)x != null) {
               return 92;
            }
         } catch (Exception var3) {
         }

         try {
            if ((Timestamp)x != null) {
               return 93;
            }
         } catch (Exception var2) {
         }

         return 1111;
      }
   }

   public synchronized void clearParameters() throws SQLException {
      if (super.binds_in.data != null && super.binds_in.data.length != 0) {
         for(int col_index = 0; col_index < super.binds_in.types.length; ++col_index) {
            super.binds_in.setItem((DBItem)null, col_index, super.current_rank);
         }

         this.clear_last_binds();
      }

   }

   public synchronized void close() throws SQLException {
      if (!super.closed) {
         this.sendBatch();
         super.close();
      }
   }
}
