package oracle.jdbc.driver;

import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.sql.Date;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Time;
import java.sql.Timestamp;
import oracle.jdbc.dbaccess.DBError;

public class OracleResultSet implements ResultSet {
   public static final boolean DEBUG = false;
   OracleConnection connection;
   OracleStatement statement;
   boolean closed;
   boolean explicitly_closed;
   public boolean close_statement_on_close;

   OracleResultSet(OracleConnection c, OracleStatement s) throws SQLException {
      this.connection = c;
      this.statement = s;
      this.close_statement_on_close = false;
      this.closed = false;
      this.explicitly_closed = false;
   }

   public synchronized boolean next() throws SQLException {
      this.connection.trace("ResultSet.next");
      if (this.explicitly_closed) {
         DBError.check_error(-10, "next");
      }

      if (this.closed) {
         return false;
      } else {
         ++this.statement.current_row;
         ++this.statement.total_rows_visited;
         if (this.statement.max_rows != 0 && this.statement.total_rows_visited > this.statement.max_rows) {
            this.internal_close();
            return false;
         } else {
            if (this.statement.current_row >= this.statement.valid_rows) {
               try {
                  if (this.statement.got_last_batch) {
                     this.internal_close();
                     return false;
                  }

                  this.statement.check_row_prefetch_changed();
                  this.prepare_for_new_row();
                  this.statement.connection.needLine();
                  this.statement.valid_rows = this.connection.db_access.fetch(this.statement.dbstmt, this.statement.defines.types, this.statement.defines.data, this.statement.row_prefetch);
                  if (this.statement.valid_rows == 0) {
                     this.internal_close();
                     return false;
                  }

                  this.statement.current_row = 0;
                  this.statement.checkValidRowsStatus();
                  return true;
               } catch (IOException e) {
                  DBError.check_error(e);
               }
            }

            return true;
         }
      }
   }

   public synchronized void close() throws SQLException {
      this.internal_close();
      this.explicitly_closed = true;
   }

   void internal_close() throws SQLException {
      this.connection.trace("ResultSet.close");
      if (!this.closed) {
         this.closed = true;

         try {
            this.statement.connection.needLine();
            this.statement.connection.db_access.closeQuery(this.statement.dbstmt);
         } catch (IOException e) {
            DBError.check_error(e);
         }

         this.statement.end_of_result_set();
         if (this.close_statement_on_close) {
            this.statement.close();
         }

      }
   }

   public synchronized boolean wasNull() throws SQLException {
      return this.statement.wasNullValue();
   }

   public synchronized String getString(int columnIndex) throws SQLException {
      return this.statement.getStringValue(true, columnIndex);
   }

   public synchronized ResultSet getCursor(int columnIndex) throws SQLException {
      return this.statement.getCursorValue(true, columnIndex);
   }

   public synchronized OracleRowid getRowid(int columnIndex) throws SQLException {
      return this.statement.getRowidValue(true, columnIndex);
   }

   public synchronized OracleBlob getBlob(int columnIndex) throws SQLException {
      return this.statement.getBlobValue(true, columnIndex);
   }

   public synchronized OracleClob getClob(int columnIndex) throws SQLException {
      return this.statement.getClobValue(true, columnIndex);
   }

   public synchronized OracleBfile getBfile(int columnIndex) throws SQLException {
      return this.statement.getBfileValue(true, columnIndex);
   }

   public synchronized OracleCfile getCfile(int columnIndex) throws SQLException {
      return this.statement.getCfileValue(true, columnIndex);
   }

   public synchronized boolean getBoolean(int columnIndex) throws SQLException {
      return this.statement.getBooleanValue(true, columnIndex);
   }

   public synchronized byte getByte(int columnIndex) throws SQLException {
      return this.statement.getByteValue(true, columnIndex);
   }

   public synchronized short getShort(int columnIndex) throws SQLException {
      return this.statement.getShortValue(true, columnIndex);
   }

   public synchronized int getInt(int columnIndex) throws SQLException {
      return this.statement.getIntValue(true, columnIndex);
   }

   public synchronized long getLong(int columnIndex) throws SQLException {
      return this.statement.getLongValue(true, columnIndex);
   }

   public synchronized float getFloat(int columnIndex) throws SQLException {
      return this.statement.getFloatValue(true, columnIndex);
   }

   public synchronized double getDouble(int columnIndex) throws SQLException {
      return this.statement.getDoubleValue(true, columnIndex);
   }

   public synchronized BigDecimal getBigDecimal(int columnIndex, int scale) throws SQLException {
      return this.statement.getBigDecimalValue(true, columnIndex, scale);
   }

   public synchronized byte[] getBytes(int columnIndex) throws SQLException {
      return this.statement.getBytesValue(true, columnIndex);
   }

   public synchronized Date getDate(int columnIndex) throws SQLException {
      return this.statement.getDateValue(true, columnIndex);
   }

   public synchronized Time getTime(int columnIndex) throws SQLException {
      return this.statement.getTimeValue(true, columnIndex);
   }

   public synchronized Timestamp getTimestamp(int columnIndex) throws SQLException {
      return this.statement.getTimestampValue(true, columnIndex);
   }

   public synchronized InputStream getAsciiStream(int columnIndex) throws SQLException {
      return this.statement.getAsciiStreamValue(true, columnIndex);
   }

   public synchronized InputStream getUnicodeStream(int columnIndex) throws SQLException {
      return this.statement.getUnicodeStreamValue(true, columnIndex);
   }

   public synchronized InputStream getBinaryStream(int columnIndex) throws SQLException {
      return this.statement.getBinaryStreamValue(true, columnIndex);
   }

   public synchronized Object getObject(int columnIndex) throws SQLException {
      return this.statement.getObjectValue(true, columnIndex);
   }

   public String getString(String columnName) throws SQLException {
      this.connection.trace("ResultSet.getString");
      return this.getString(this.findColumn(columnName));
   }

   public OracleRowid getRowid(String columnName) throws SQLException {
      this.connection.trace("ResultSet.getRowid");
      return this.getRowid(this.findColumn(columnName));
   }

   public OracleBlob getBlob(String columnName) throws SQLException {
      this.connection.trace("ResultSet.getBlob");
      return this.getBlob(this.findColumn(columnName));
   }

   public OracleClob getClob(String columnName) throws SQLException {
      this.connection.trace("ResultSet.getClob");
      return this.getClob(this.findColumn(columnName));
   }

   public OracleBfile getBfile(String columnName) throws SQLException {
      this.connection.trace("ResultSet.getBfile");
      return this.getBfile(this.findColumn(columnName));
   }

   public OracleCfile getCfile(String columnName) throws SQLException {
      this.connection.trace("ResultSet.getCfile");
      return this.getCfile(this.findColumn(columnName));
   }

   public boolean getBoolean(String columnName) throws SQLException {
      this.connection.trace("ResultSet.getBoolean");
      return this.getBoolean(this.findColumn(columnName));
   }

   public byte getByte(String columnName) throws SQLException {
      this.connection.trace("ResultSet.getByte");
      return this.getByte(this.findColumn(columnName));
   }

   public short getShort(String columnName) throws SQLException {
      this.connection.trace("ResultSet.getShort");
      return this.getShort(this.findColumn(columnName));
   }

   public int getInt(String columnName) throws SQLException {
      this.connection.trace("ResultSet.getInt");
      return this.getInt(this.findColumn(columnName));
   }

   public long getLong(String columnName) throws SQLException {
      this.connection.trace("ResultSet.getLong");
      return this.getLong(this.findColumn(columnName));
   }

   public float getFloat(String columnName) throws SQLException {
      this.connection.trace("ResultSet.getFloat");
      return this.getFloat(this.findColumn(columnName));
   }

   public double getDouble(String columnName) throws SQLException {
      this.connection.trace("ResultSet.getDouble");
      return this.getDouble(this.findColumn(columnName));
   }

   public BigDecimal getBigDecimal(String columnName, int scale) throws SQLException {
      this.connection.trace("ResultSet.getBigDecimal");
      return this.getBigDecimal(this.findColumn(columnName), scale);
   }

   public byte[] getBytes(String columnName) throws SQLException {
      this.connection.trace("ResultSet.getBytes");
      return this.getBytes(this.findColumn(columnName));
   }

   public Date getDate(String columnName) throws SQLException {
      this.connection.trace("ResultSet.getDate");
      return this.getDate(this.findColumn(columnName));
   }

   public Time getTime(String columnName) throws SQLException {
      this.connection.trace("ResultSet.getTime");
      return this.getTime(this.findColumn(columnName));
   }

   public Timestamp getTimestamp(String columnName) throws SQLException {
      this.connection.trace("ResultSet.getTimestamp");
      return this.getTimestamp(this.findColumn(columnName));
   }

   public InputStream getAsciiStream(String columnName) throws SQLException {
      this.connection.trace("ResultSet.getAsciiStream");
      return this.getAsciiStream(this.findColumn(columnName));
   }

   public InputStream getUnicodeStream(String columnName) throws SQLException {
      this.connection.trace("ResultSet.getUnicodeStream");
      return this.getUnicodeStream(this.findColumn(columnName));
   }

   public InputStream getBinaryStream(String columnName) throws SQLException {
      this.connection.trace("ResultSet.getBinaryStream");
      return this.getBinaryStream(this.findColumn(columnName));
   }

   public Object getObject(String columnName) throws SQLException {
      this.connection.trace("ResultSet.getObject");
      return this.getObject(this.findColumn(columnName));
   }

   public SQLWarning getWarnings() throws SQLException {
      return null;
   }

   public void clearWarnings() throws SQLException {
      this.connection.trace("ResultSet.getWarnings");
   }

   public String getCursorName() throws SQLException {
      this.connection.trace("ResultSet.getCursorName");
      DBError.check_error(-23, "getCursorName");
      return null;
   }

   public synchronized ResultSetMetaData getMetaData() throws SQLException {
      this.connection.trace("ResultSet.getMetaData");
      if (this.explicitly_closed) {
         DBError.check_error(-10, "getMetaData");
      }

      return new OracleResultSetMetaData(this.connection, this.statement);
   }

   public synchronized int findColumn(String columnName) throws SQLException {
      this.connection.trace("ResultSet.findColumn");
      return this.statement.get_column_index(columnName);
   }

   void prepare_for_new_row() throws SQLException {
      if (this.statement.streams != null) {
         try {
            this.statement.streams.close();
         } catch (IOException e) {
            DBError.check_error(e);
         }
      }

      this.clearWarnings();
   }
}
