package oracle.jdbc.driver;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.sql.Date;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Statement;
import java.sql.Time;
import java.sql.Timestamp;
import oracle.jdbc.dbaccess.DBAccess;
import oracle.jdbc.dbaccess.DBColumn;
import oracle.jdbc.dbaccess.DBConversion;
import oracle.jdbc.dbaccess.DBDataSet;
import oracle.jdbc.dbaccess.DBError;
import oracle.jdbc.dbaccess.DBItem;
import oracle.jdbc.dbaccess.DBStatement;
import oracle.jdbc.dbaccess.DBType;
import oracle.jdbc.util.RepConversion;

public class OracleStatement implements Statement {
   static final boolean DEBUG = false;
   static final boolean DEBUG_DATASET = false;
   static final int LONG_SIZE = Integer.MAX_VALUE;
   DBColumn[] dbdescription;
   DBDataSet binds_in;
   DBDataSet binds_out;
   public DBDataSet defines;
   OracleConnection connection;
   OracleInputStream streams;
   InputStream[] stream_array;
   OracleResultSet current_result_set;
   boolean process_escapes;
   int query_timeout;
   public DBStatement dbstmt;
   int batch;
   int row_prefetch;
   int current_rank;
   int current_row;
   int valid_rows;
   int max_field_size;
   int max_rows;
   int total_rows_visited;
   boolean got_last_batch;
   int new_row_prefetch;
   boolean closed;
   boolean need_to_parse;
   boolean need_to_prepare_define_buffer;
   boolean columns_defined_by_user;
   byte sql_kind;
   byte[] array_sql;
   public int auto_rollback;
   public int wait_option;
   boolean last_was_column;
   int last_index;

   void initialize(int batch_value, int row_prefetch_value) {
      this.connection.add_statement(this);
      this.current_result_set = null;
      this.streams = null;
      this.last_index = 0;
      this.process_escapes = true;
      this.query_timeout = 0;
      this.auto_rollback = 2;
      this.wait_option = 0;
      this.batch = batch_value;
      this.row_prefetch = row_prefetch_value;
      this.current_rank = 0;
      this.current_row = 0;
      this.valid_rows = 0;
      this.max_rows = 0;
      this.total_rows_visited = 0;
      this.max_field_size = 0;
      this.got_last_batch = false;
      this.binds_in = new DBDataSet(this.batch);
      this.binds_out = new DBDataSet(0);
      this.defines = new DBDataSet(0);
      this.stream_array = null;
      this.closed = false;
      this.need_to_parse = true;
      this.need_to_prepare_define_buffer = true;
      this.columns_defined_by_user = false;
   }

   public OracleStatement(OracleConnection c, int batch_value, int row_prefetch_value) throws SQLException {
      this.connection = c;

      try {
         this.connection.needLine();
         this.dbstmt = this.connection.db_access.open();
      } catch (IOException e) {
         DBError.check_error(e);
      }

      this.initialize(batch_value, row_prefetch_value);
   }

   public OracleStatement(OracleConnection c, DBStatement cursor_dbstmt, int batch_value, int row_prefetch_value) throws SQLException {
      this.connection = c;
      DBAccess db = this.connection.db_access;
      this.dbstmt = cursor_dbstmt;
      this.initialize(batch_value, row_prefetch_value);
      this.prepare_for_new_result();
      this.current_result_set = new OracleResultSet(this.connection, this);
      this.current_result_set.close_statement_on_close = true;

      try {
         this.connection.needLine();
         this.dbdescription = this.connection.db_access.describe(this.dbstmt);
         this.need_to_parse = false;
         this.doDefaultTypes();
         this.doDefinesFromTypes();
         this.need_to_prepare_define_buffer = false;
         this.connection.needLine();
         this.valid_rows = db.fetch(this.dbstmt, this.defines.types, this.defines.data, this.row_prefetch);
      } catch (IOException e) {
         DBError.check_error(e);
      }

      this.checkValidRowsStatus();
   }

   public synchronized ResultSet executeQuery(String sql) throws SQLException {
      this.connection.trace("Statement.executeQuery");
      this.sendBatch();
      this.need_to_parse = true;
      this.prepare_for_new_result();
      this.doExecuteWithTimeout(sql);
      this.current_result_set = new OracleResultSet(this.connection, this);
      return this.current_result_set;
   }

   byte getSqlKind(String sql) {
      String clean_sql = sql.trim();
      if (clean_sql.regionMatches(true, 0, "select", 0, 6)) {
         return 0;
      } else {
         return (byte)(clean_sql.regionMatches(true, 0, "begin", 0, 5) ? 1 : 2);
      }
   }

   void doDefaultTypes() throws SQLException {
      OracleInputStream current_stream = null;
      int column_num = this.dbdescription.length;
      this.defines.types = new DBType[column_num];

      for(int column_index = 0; column_index < column_num; ++column_index) {
         DBColumn column_info = this.dbdescription[column_index];
         switch (column_info.type) {
            case 1:
            case 96:
               int size = column_info.max_length;
               if (this.max_field_size != 0) {
                  size = Math.min(this.max_field_size, size);
               }

               short char_set = this.connection.conversion.getCharacterSet();
               if (char_set != 1 && char_set != 31) {
                  size = 3 * size;
               }

               this.defines.setType(column_index, new DBType(column_info.type, size));
               break;
            case 2:
            case 6:
               this.defines.setType(column_index, new DBType(6));
               break;
            case 8:
               int size = Integer.MAX_VALUE;
               if (this.max_field_size != 0) {
                  size = Math.min(this.max_field_size, size);
               }

               this.defines.setType(column_index, new DBType(8, size, true));
               break;
            case 11:
               this.defines.setType(column_index, new DBType(1, 64));
               break;
            case 12:
               this.defines.setType(column_index, new DBType(column_info.type));
               break;
            case 23:
               int size = column_info.max_length;
               if (this.max_field_size != 0) {
                  size = Math.min(this.max_field_size, size);
               }

               this.defines.setType(column_index, new DBType(column_info.type, size));
               break;
            case 24:
               int size = Integer.MAX_VALUE;
               if (this.max_field_size != 0) {
                  size = Math.min(this.max_field_size, size);
               }

               this.defines.setType(column_index, new DBType(24, size, true));
               break;
            case 102:
               int cursor_size = this.connection.db_access.getRefCursorBytesSize();
               this.defines.setType(column_index, new DBType(102, cursor_size));
               break;
            default:
               this.defines.setType(column_index, new DBType(column_info.type, column_info.max_length));
         }
      }

   }

   void doDefinesFromTypes() throws SQLException {
      this.streams = null;
      int column_num = this.defines.types.length;

      for(int column_index = 0; column_index < column_num; ++column_index) {
         DBType dbtype = this.defines.types[column_index];
         if (dbtype.is_stream) {
            this.row_prefetch = 1;
            this.stream_array = new InputStream[column_num];
            break;
         }
      }

      this.defines.createArrayFromTypes(this.row_prefetch);

      for(int var6 = 0; var6 < column_num; ++var6) {
         DBType dbtype = this.defines.types[var6];
         if (dbtype.is_stream) {
            OracleInputStream stream = new OracleInputStream(this, var6 + 1, this.streams);
            DBItem item = new DBItem(dbtype.max_length);
            item.defineStreamData(stream);
            this.streams = (OracleInputStream)item.stream_value;
            this.defines.setItem(item, var6, 0);
         } else {
            DBItem item = new DBItem(dbtype.max_length);
            item.allocBuffer(dbtype.max_length);
            this.defines.setRowItems(var6, item);
         }
      }

      if (this.stream_array != null) {
         this.stream_array[0] = null;

         for(int var7 = 1; var7 < column_num; ++var7) {
            if (!this.defines.types[var7 - 1].is_stream) {
               this.stream_array[var7] = this.stream_array[var7 - 1];
            } else {
               this.stream_array[var7] = this.defines.data[var7 - 1].items[0].stream_value;
            }
         }
      }

   }

   public synchronized void clearDefines() throws SQLException {
      this.columns_defined_by_user = false;
      this.need_to_prepare_define_buffer = true;
      this.defines = new DBDataSet(0);
   }

   public synchronized void defineColumnType(int column_index, int type) throws SQLException {
      if (this.current_result_set != null && !this.current_result_set.closed) {
         DBError.check_error(-28, "defineColumnType");
      }

      if (!this.columns_defined_by_user) {
         this.clearDefines();
         this.columns_defined_by_user = true;
      }

      this.need_to_prepare_define_buffer = true;
      DBType dbtype = null;
      switch (type) {
         case -14:
            dbtype = new DBType(115);
            break;
         case -13:
            dbtype = new DBType(114);
            break;
         case -12:
            dbtype = new DBType(112);
            break;
         case -11:
            dbtype = new DBType(113);
            break;
         case -10:
            int cursor_size = this.connection.db_access.getRefCursorBytesSize();
            dbtype = new DBType(102, cursor_size);
            break;
         case -8:
            dbtype = new DBType(11);
            break;
         case -7:
         case -6:
         case -5:
         case 2:
         case 3:
         case 4:
         case 5:
         case 6:
         case 7:
         case 8:
            dbtype = new DBType(6);
            break;
         case -4:
            dbtype = new DBType(24, Integer.MAX_VALUE, true);
            break;
         case -3:
         case -2:
            dbtype = new DBType(23);
            break;
         case -1:
            dbtype = new DBType(8, Integer.MAX_VALUE, true);
            break;
         case 1:
            dbtype = new DBType(96);
            break;
         case 12:
            dbtype = new DBType(1);
            break;
         case 91:
         case 92:
         case 93:
            dbtype = new DBType(12);
      }

      if (this.max_field_size != 0) {
         dbtype.max_length = Math.min(this.max_field_size, this.defines.types[column_index - 1].max_length);
      }

      if (this.defines.types == null) {
         this.defines.types = new DBType[column_index];
         this.defines.types[column_index - 1] = dbtype;
      } else if (this.defines.types.length < column_index) {
         DBType[] prev_types = this.defines.types;
         this.defines.types = new DBType[column_index];
         System.arraycopy(prev_types, 0, this.defines.types, 0, prev_types.length);
         this.defines.types[column_index - 1] = dbtype;
      } else {
         this.defines.types[column_index - 1] = dbtype;
      }

   }

   public synchronized void defineColumnType(int column_index, int type, int max_size) throws SQLException {
      this.defineColumnType(column_index, type);
      int max_len;
      if (this.max_field_size != 0) {
         max_len = Math.min(max_size, this.max_field_size);
      } else {
         max_len = max_size;
      }

      max_len = Math.min(this.defines.types[column_index - 1].max_length, max_len);
      this.defines.types[column_index - 1].max_length = max_len;
   }

   public synchronized void setRowPrefetch(int value) throws SQLException {
      if (value <= 0) {
         DBError.check_error(-20, "setRowPrefetch");
      }

      this.new_row_prefetch = value;
   }

   public int getRowPrefetch() {
      return this.new_row_prefetch != 0 ? this.new_row_prefetch : this.row_prefetch;
   }

   void checkBindsInAndOut() throws SQLException {
      int length = 0;
      if (this.binds_in.types != null) {
         length = this.binds_in.types.length;
      }

      if (this.binds_out.types != null && this.binds_out.types.length > length) {
         length = this.binds_out.types.length;
      }

      for(int i = 0; i < length; ++i) {
         if ((this.binds_in.data == null || i >= this.binds_in.data.length || this.binds_in.data[i] == null || this.binds_in.data[i].items == null || this.binds_in.data[i].items[this.current_rank] == null) && (this.binds_out.data == null || i >= this.binds_out.data.length || this.binds_out.data[i] == null || this.binds_out.data[i].items == null || this.binds_out.data[i].items[this.current_rank] == null)) {
            DBError.check_error(-41, new Integer(i + 1));
         }
      }

   }

   void doExecuteOther() throws SQLException {
      DBAccess db = this.connection.db_access;

      try {
         this.connection.needLine();
         if (this.need_to_parse) {
            this.valid_rows = db.parseExecuteFetch(this.dbstmt, this.sql_kind, this.array_sql, this.binds_in.types, this.binds_in.data, this.current_rank, this.binds_out.types, this.binds_out.data, this.current_rank);
            this.need_to_parse = false;
         } else {
            this.valid_rows = db.executeFetch(this.dbstmt, this.sql_kind, this.binds_in.types, this.binds_in.data, this.current_rank, this.binds_out.types, this.binds_out.data, this.current_rank);
         }
      } catch (IOException e) {
         DBError.check_error(e);
      }

      this.checkValidRowsStatus();
   }

   void doExecuteWithBatch() throws SQLException {
      ++this.current_rank;
      if (this.current_rank >= this.batch) {
         try {
            this.doExecuteOther();
         } finally {
            this.current_rank = 0;
         }
      }

   }

   void check_defines() throws SQLException {
      if (this.defines.types == null) {
         DBError.check_error(-21, "check_defines");
      }

      for(int index = 0; index < this.defines.types.length; ++index) {
         if (this.defines.types[index] == null) {
            DBError.check_error(-22, new Integer(index + 1));
         }
      }

   }

   void check_row_prefetch_changed() {
      if (this.defines.types != null) {
         int column_num = this.defines.types.length;

         for(int column_index = 0; column_index < column_num; ++column_index) {
            if (this.defines.types[column_index].is_stream) {
               this.new_row_prefetch = 0;
               break;
            }
         }
      }

      if (this.new_row_prefetch != 0) {
         if (this.defines.data == null) {
            this.row_prefetch = this.new_row_prefetch;
            this.new_row_prefetch = 0;
         } else if (this.new_row_prefetch != this.row_prefetch) {
            DBDataSet new_defines = new DBDataSet(this.defines.types.length, this.new_row_prefetch);

            for(int index = 0; index < this.defines.types.length; ++index) {
               new_defines.types[index] = this.defines.types[index];
               new_defines.setRowItems(index, this.defines.data[index].items[0]);
            }

            this.defines = new_defines;
            this.row_prefetch = this.new_row_prefetch;
            this.new_row_prefetch = 0;
         } else {
            this.row_prefetch = this.new_row_prefetch;
            this.new_row_prefetch = 0;
         }
      }

   }

   void doExecuteQuery() throws SQLException {
      DBAccess db = this.connection.db_access;

      try {
         this.connection.needLine();
         if (this.need_to_parse) {
            if (this.need_to_prepare_define_buffer) {
               if (this.columns_defined_by_user) {
                  this.check_defines();
                  this.check_row_prefetch_changed();
                  this.doDefinesFromTypes();
                  this.need_to_prepare_define_buffer = false;
                  this.valid_rows = db.parseExecuteFetch(this.dbstmt, this.sql_kind, this.array_sql, this.binds_in.types, this.binds_in.data, 1, this.defines.types, this.defines.data, this.row_prefetch);
                  this.need_to_parse = false;
               } else {
                  this.dbdescription = db.parseExecuteDescribe(this.dbstmt, this.array_sql, this.binds_in.types, this.binds_in.data);
                  this.need_to_parse = false;
                  this.doDefaultTypes();
                  this.check_row_prefetch_changed();
                  this.doDefinesFromTypes();
                  this.need_to_prepare_define_buffer = false;
                  this.valid_rows = db.fetch(this.dbstmt, this.defines.types, this.defines.data, this.row_prefetch);
               }
            } else {
               this.check_row_prefetch_changed();
               this.valid_rows = db.parseExecuteFetch(this.dbstmt, this.sql_kind, this.array_sql, this.binds_in.types, this.binds_in.data, 1, this.defines.types, this.defines.data, this.row_prefetch);
            }
         } else if (this.need_to_prepare_define_buffer) {
            if (this.columns_defined_by_user) {
               this.check_defines();
               this.check_row_prefetch_changed();
               this.doDefinesFromTypes();
               this.need_to_prepare_define_buffer = false;
               this.valid_rows = db.executeFetch(this.dbstmt, this.sql_kind, this.binds_in.types, this.binds_in.data, 1, this.defines.types, this.defines.data, this.row_prefetch);
               this.need_to_parse = false;
            } else {
               this.dbdescription = db.parseExecuteDescribe(this.dbstmt, this.array_sql, this.binds_in.types, this.binds_in.data);
               this.need_to_parse = false;
               this.doDefaultTypes();
               this.check_row_prefetch_changed();
               this.doDefinesFromTypes();
               this.need_to_prepare_define_buffer = false;
               this.valid_rows = db.fetch(this.dbstmt, this.defines.types, this.defines.data, this.row_prefetch);
            }
         } else {
            this.check_row_prefetch_changed();
            this.valid_rows = db.executeFetch(this.dbstmt, this.sql_kind, this.binds_in.types, this.binds_in.data, 1, this.defines.types, this.defines.data, this.row_prefetch);
         }
      } catch (IOException e) {
         DBError.check_error(e);
      }

      this.checkValidRowsStatus();
   }

   void checkValidRowsStatus() {
      if (this.valid_rows == -2) {
         this.valid_rows = 1;
         this.connection.holdLine(this);
         if (this.streams != null) {
            this.streams.resetLong();
         }
      } else if (this.sql_kind == 0 && this.valid_rows < this.row_prefetch) {
         this.got_last_batch = true;
      }

   }

   void doExecute() throws SQLException {
      if (this.sql_kind == 0) {
         this.doExecuteQuery();
      } else {
         this.doExecuteWithBatch();
      }

   }

   void doExecuteWithTimeout(String sql) throws SQLException {
      this.checkBindsInAndOut();
      if (this.need_to_parse) {
         String oracle_sql = this.process_escapes ? this.connection.nativeSQL(sql) : sql;
         this.sql_kind = this.getSqlKind(oracle_sql);
         this.array_sql = this.connection.conversion.StringToCharBytes(oracle_sql);
      }

      if (this.query_timeout != 0) {
         OracleCancelThread cancel_thread = null;

         try {
            cancel_thread = new OracleCancelThread(this, this.query_timeout);
            ((Thread)cancel_thread).start();
            this.doExecute();
         } finally {
            cancel_thread.stopMe();
         }
      } else {
         this.doExecute();
      }

   }

   public synchronized int executeUpdate(String sql) throws SQLException {
      this.connection.trace("Statement.executeUpdate");
      this.sendBatch();
      this.need_to_parse = true;
      this.prepare_for_new_result();
      this.doExecuteWithTimeout(sql);
      return this.valid_rows;
   }

   public synchronized void close() throws SQLException {
      this.connection.trace("Statement.close");
      if (!this.closed) {
         this.closed = true;
         this.connection.remove_statement(this);

         try {
            this.connection.needLine();
            this.connection.db_access.close(this.dbstmt);
         } catch (IOException e) {
            DBError.check_error(e);
         }

      }
   }

   public int getMaxFieldSize() throws SQLException {
      return this.max_field_size;
   }

   public synchronized void setMaxFieldSize(int max) throws SQLException {
      this.max_field_size = max;
   }

   public int getMaxRows() throws SQLException {
      return this.max_rows;
   }

   public synchronized void setMaxRows(int max) throws SQLException {
      this.max_rows = max;
   }

   public synchronized void setEscapeProcessing(boolean enable) throws SQLException {
      this.connection.trace("Statement.setEscapeProcessing");
      if (enable) {
         this.process_escapes = true;
      } else {
         this.process_escapes = false;
      }

   }

   public int getQueryTimeout() throws SQLException {
      this.connection.trace("Statement.getQueryTimeout");
      return this.query_timeout;
   }

   public synchronized void setQueryTimeout(int max) throws SQLException {
      this.connection.trace("Statement.setQueryTimeout");
      this.query_timeout = max;
   }

   public void cancel() throws SQLException {
      try {
         this.connection.db_access.cancel();
         this.connection.releaseLine();
      } catch (IOException e) {
         DBError.check_error(e);
      }

   }

   public synchronized void setAutoRollback(int autoRollback) throws SQLException {
      this.connection.trace("Statement.setAutoRollback");
      this.connection.needLine();
      this.connection.db_access.setWaitandAutoRollback(this.dbstmt, this.wait_option, autoRollback);
      this.auto_rollback = autoRollback;
   }

   public synchronized int getAutoRollback() throws SQLException {
      this.connection.trace("Statement.getAutoRollback");
      return this.auto_rollback;
   }

   public synchronized void setWaitOption(int waitOption) throws SQLException {
      this.connection.trace("Statement.setWaitOption");
      this.connection.needLine();
      this.connection.db_access.setWaitandAutoRollback(this.dbstmt, waitOption, this.auto_rollback);
      this.wait_option = waitOption;
   }

   public synchronized int getWaitOption() throws SQLException {
      this.connection.trace("Connection.getWaitOption");
      return this.wait_option;
   }

   public SQLWarning getWarnings() throws SQLException {
      this.connection.trace("Statement.getWarnings");
      return null;
   }

   public void clearWarnings() throws SQLException {
      this.connection.trace("Statement.clearWarnings");
   }

   public void setCursorName(String name) throws SQLException {
      this.connection.trace("Statement.setCursorName");
      DBError.check_error(-23, "setCursorName");
   }

   public boolean wasSelect(String sql) {
      return this.sql_kind == 0;
   }

   public synchronized boolean execute(String sql) throws SQLException {
      this.connection.trace("Statement.execute");
      this.sendBatch();
      this.need_to_parse = true;
      this.prepare_for_new_result();
      this.doExecuteWithTimeout(sql);
      return this.sql_kind == 0;
   }

   public synchronized ResultSet getResultSet() throws SQLException {
      this.connection.trace("Statement.getResultSet");
      if (this.sql_kind == 0) {
         if (this.current_result_set == null) {
            this.current_result_set = new OracleResultSet(this.connection, this);
         }

         return this.current_result_set;
      } else {
         return null;
      }
   }

   public int getUpdateCount() throws SQLException {
      this.connection.trace("Statement.getUpdateCount");
      return this.sql_kind != 2 ? 0 : this.valid_rows;
   }

   public boolean getMoreResults() throws SQLException {
      this.connection.trace("Statement.getMoreResults");
      return false;
   }

   public synchronized int sendBatch() throws SQLException {
      return 0;
   }

   void prepare_for_new_result() throws SQLException {
      this.clearWarnings();
      if (this.streams != null) {
         try {
            this.streams.close();
         } catch (IOException e) {
            DBError.check_error(e);
         }

         this.streams = null;
      }

      if (this.current_result_set != null) {
         this.current_result_set.internal_close();
         this.current_result_set = null;
      }

      this.current_row = -1;
      this.valid_rows = 0;
      this.total_rows_visited = 0;
      this.got_last_batch = false;
      if (!this.columns_defined_by_user && this.need_to_parse) {
         this.need_to_prepare_define_buffer = true;
      }

      if (this.need_to_parse) {
         this.dbdescription = null;
      }

   }

   void end_of_result_set() throws SQLException {
      this.prepare_for_new_result();
      this.clearDefines();
   }

   boolean wasNullValue() throws SQLException {
      this.connection.trace("Statement.wasNull");
      if (this.last_index == 0) {
         DBError.check_error(-24, "wasNullValue");
      }

      return this.sql_kind == 0 ? this.defines.isNull(this.last_index - 1, this.current_row) : this.binds_out.isNull(this.last_index - 1, this.current_rank);
   }

   void prepare_for_new_get(boolean getColumn, int column_index) throws SQLException {
      if (getColumn) {
         if (this.current_result_set == null || this.current_result_set.closed) {
            DBError.check_error(-11, "prepare_for_new_get");
         }

         if (this.defines == null || column_index > this.defines.data.length || column_index <= 0) {
            DBError.check_error(-3, "prepare_for_new_get");
         }

         if (this.current_row < 0) {
            DBError.check_error(-14, "prepare_for_new_get");
         }
      } else if (this.binds_out == null || column_index > this.binds_out.data.length || column_index <= 0 || this.binds_out.data[column_index - 1] == null) {
         DBError.check_error(-3, "prepare_for_new_get");
      }

      if (this.stream_array != null && this.stream_array[column_index - 1] != null) {
         try {
            this.stream_array[column_index - 1].close();
         } catch (IOException e) {
            DBError.check_error(e);
         }
      }

      this.last_was_column = getColumn;
      this.last_index = column_index;
   }

   byte[] getBytesInternal(boolean getColumn, int index) throws SQLException {
      this.connection.trace("Statement.getBytesInternal");
      --index;
      DBType dbtype;
      DBItem item;
      if (getColumn) {
         dbtype = this.defines.getType(index);
         item = this.defines.getData(index).getItem(this.current_row);
      } else {
         dbtype = this.binds_out.getType(index);
         item = this.binds_out.getData(index).getItem(this.current_rank);
      }

      if (!dbtype.is_stream) {
         int data_size = item.data_size;
         int max_out_size = item.max_out_size;
         if (data_size < max_out_size) {
            byte[] result = new byte[data_size];
            System.arraycopy(item.byte_value, 0, result, 0, data_size);
            return result;
         } else {
            return item.byte_value;
         }
      } else {
         InputStream stream = item.stream_value;
         ByteArrayOutputStream out = new ByteArrayOutputStream(1024);
         byte[] buffer = new byte[1024];

         int length;
         try {
            while((length = stream.read(buffer)) != -1) {
               out.write(buffer, 0, length);
            }
         } catch (IOException e) {
            DBError.check_error(e);
         }

         return out.toByteArray();
      }
   }

   String getStringValue(boolean getColumn, int index) throws SQLException {
      this.connection.trace("Statement.getString");
      this.prepare_for_new_get(getColumn, index);
      if (this.is_value_null(getColumn, index)) {
         return null;
      } else {
         int type = this.get_value_type(getColumn, index);
         byte[] array = this.getBytesInternal(getColumn, index);
         if (array != null && array.length != 0) {
            switch (type) {
               case 1:
               case 8:
               case 96:
                  return this.connection.conversion.CharBytesToString(array, array.length);
               case 2:
               case 6:
                  BigDecimal num = this.connection.conversion.NumberBytesToBigDecimal(array, array.length);
                  return num.toString();
               case 12:
                  Timestamp tstamp = this.connection.conversion.DateBytesToTimestamp(array, array.length);
                  return tstamp.toString();
               case 23:
               case 24:
                  return RepConversion.bArray2String(array);
               default:
                  return null;
            }
         } else {
            return null;
         }
      }
   }

   ResultSet getCursorValue(boolean getColumn, int index) throws SQLException {
      this.connection.trace("Statement.getCursorValue");
      this.prepare_for_new_get(getColumn, index);
      if (this.is_value_null(getColumn, index)) {
         return null;
      } else {
         DBAccess db = this.connection.db_access;
         byte[] result = this.getBytesInternal(getColumn, index);
         if (result != null && result.length != 0) {
            DBStatement db_stmt = null;

            try {
               db_stmt = db.RefCursorBytesToDBStatement(result);
            } catch (IOException e) {
               DBError.check_error(e);
            }

            OracleStatement stmt = new OracleStatement(this.connection, db_stmt, 1, 1);
            return stmt.getResultSet();
         } else {
            return null;
         }
      }
   }

   OracleRowid getRowidValue(boolean getColumn, int index) throws SQLException {
      this.connection.trace("Statement.getRowid");
      this.prepare_for_new_get(getColumn, index);
      return this.is_value_null(getColumn, index) ? null : new OracleRowid(this, this.getStringValue(getColumn, index));
   }

   OracleBlob getBlobValue(boolean getColumn, int index) throws SQLException {
      this.connection.trace("Statement.getLob");
      int type = this.get_value_type(getColumn, index);
      if (type != 113) {
         DBError.check_error(-4, "getBlob");
      }

      byte[] lob_descriptor = this.getBytesInternal(getColumn, index);
      return lob_descriptor == null ? null : new OracleBlob(lob_descriptor);
   }

   OracleClob getClobValue(boolean getColumn, int index) throws SQLException {
      this.connection.trace("Statement.getLob");
      int type = this.get_value_type(getColumn, index);
      if (type != 112) {
         DBError.check_error(-4, "getLob");
      }

      byte[] lob_descriptor = this.getBytesInternal(getColumn, index);
      return lob_descriptor == null ? null : new OracleClob(lob_descriptor);
   }

   OracleBfile getBfileValue(boolean getColumn, int index) throws SQLException {
      this.connection.trace("Statement.getFile");
      int type = this.get_value_type(getColumn, index);
      if (type != 114) {
         DBError.check_error(-4, "getBfile");
      }

      byte[] file_descriptor = this.getBytesInternal(getColumn, index);
      return file_descriptor == null ? null : new OracleBfile(file_descriptor);
   }

   OracleCfile getCfileValue(boolean getColumn, int index) throws SQLException {
      this.connection.trace("Statement.getFile");
      int type = this.get_value_type(getColumn, index);
      if (type != 115) {
         DBError.check_error(-4, "getFile");
      }

      byte[] file_descriptor = this.getBytesInternal(getColumn, index);
      return file_descriptor == null ? null : new OracleCfile(file_descriptor);
   }

   boolean getBooleanValue(boolean getColumn, int index) throws SQLException {
      BigDecimal result = this.getBigDecimalValue(getColumn, index, 0);
      if (result != null) {
         return result.signum() != 0;
      } else {
         return false;
      }
   }

   byte getByteValue(boolean getColumn, int index) throws SQLException {
      long result = this.getLongValue(getColumn, index);
      if (result > 127L || result < -128L) {
         DBError.check_error(-26, "getByteValue");
      }

      return (byte)((int)result);
   }

   short getShortValue(boolean getColumn, int index) throws SQLException {
      long result = this.getLongValue(getColumn, index);
      if (result > 65537L || result < -65538L) {
         DBError.check_error(-26, "getShortValue");
      }

      return (short)((int)result);
   }

   int getIntValue(boolean getColumn, int index) throws SQLException {
      long result = this.getLongValue(getColumn, index);
      if (result > 2147483647L || result < -2147483648L) {
         DBError.check_error(-26, "getIntValue");
      }

      return (int)result;
   }

   long getLongValue(boolean getColumn, int index) throws SQLException {
      this.connection.trace("Statement.getLong");
      this.prepare_for_new_get(getColumn, index);
      if (this.is_value_null(getColumn, index)) {
         return 0L;
      } else {
         byte[] result = this.getBytesInternal(getColumn, index);
         if (result != null && result.length != 0) {
            int type = this.get_value_type(getColumn, index);
            switch (type) {
               case 1:
               case 8:
               case 96:
                  String str = this.connection.conversion.CharBytesToString(result, result.length);
                  BigDecimal num = new BigDecimal(str);
                  return num.longValue();
               case 2:
               case 6:
                  return this.connection.conversion.NumberBytesToLong(result, result.length);
               default:
                  DBError.check_error(-4, "getLong");
                  return 0L;
            }
         } else {
            return 0L;
         }
      }
   }

   float getFloatValue(boolean getColumn, int index) throws SQLException {
      this.connection.trace("Statement.getFloat");
      this.prepare_for_new_get(getColumn, index);
      if (this.is_value_null(getColumn, index)) {
         return 0.0F;
      } else {
         byte[] result = this.getBytesInternal(getColumn, index);
         if (result != null && result.length != 0) {
            int type = this.get_value_type(getColumn, index);
            switch (type) {
               case 1:
               case 8:
               case 96:
                  String str = this.connection.conversion.CharBytesToString(result, result.length);
                  BigDecimal num = new BigDecimal(str);
                  return num.floatValue();
               case 2:
               case 6:
                  return this.connection.conversion.NumberBytesToFloat(result, result.length);
               default:
                  DBError.check_error(-4, "getFloat");
                  return 0.0F;
            }
         } else {
            return 0.0F;
         }
      }
   }

   double getDoubleValue(boolean getColumn, int index) throws SQLException {
      this.connection.trace("Statement.getDouble");
      this.prepare_for_new_get(getColumn, index);
      if (this.is_value_null(getColumn, index)) {
         return (double)0.0F;
      } else {
         byte[] result = this.getBytesInternal(getColumn, index);
         if (result != null && result.length != 0) {
            int type = this.get_value_type(getColumn, index);
            switch (type) {
               case 1:
               case 8:
               case 96:
                  String str = this.connection.conversion.CharBytesToString(result, result.length);
                  BigDecimal num = new BigDecimal(str);
                  return num.doubleValue();
               case 2:
               case 6:
                  return this.connection.conversion.NumberBytesToDouble(result, result.length);
               default:
                  DBError.check_error(-4, "getDouble");
                  return (double)0.0F;
            }
         } else {
            return (double)0.0F;
         }
      }
   }

   BigDecimal getBigDecimalValue(boolean getColumn, int index, int scale) throws SQLException {
      this.connection.trace("Statement.getBigDecimal");
      this.prepare_for_new_get(getColumn, index);
      if (this.is_value_null(getColumn, index)) {
         return null;
      } else {
         byte[] result = this.getBytesInternal(getColumn, index);
         if (result != null && result.length != 0) {
            int type = this.get_value_type(getColumn, index);
            switch (type) {
               case 1:
               case 8:
               case 96:
                  String str = this.connection.conversion.CharBytesToString(result, result.length);
                  BigDecimal num = new BigDecimal(str);
                  return num;
               case 2:
               case 6:
                  return this.connection.conversion.NumberBytesToBigDecimal(result, result.length);
               default:
                  DBError.check_error(-4, "getBigDecimal");
                  return null;
            }
         } else {
            return null;
         }
      }
   }

   byte[] getBytesValue(boolean getColumn, int index) throws SQLException {
      this.connection.trace("Statement.getBytes");
      this.prepare_for_new_get(getColumn, index);
      return this.is_value_null(getColumn, index) ? null : this.getBytesInternal(getColumn, index);
   }

   Date getDateValue(boolean getColumn, int index) throws SQLException {
      this.connection.trace("Statement.getDate");
      this.prepare_for_new_get(getColumn, index);
      if (this.is_value_null(getColumn, index)) {
         return null;
      } else {
         byte[] result = this.getBytesInternal(getColumn, index);
         if (result != null && result.length != 0) {
            int type = this.get_value_type(getColumn, index);
            switch (type) {
               case 1:
               case 8:
               case 96:
                  String str = this.connection.conversion.CharBytesToString(result, result.length);
                  return Date.valueOf(str);
               case 12:
                  return this.connection.conversion.DateBytesToDate(result, result.length);
               default:
                  DBError.check_error(-4, "getDate");
                  return null;
            }
         } else {
            return null;
         }
      }
   }

   Time getTimeValue(boolean getColumn, int index) throws SQLException {
      this.connection.trace("Statement.getTime");
      this.prepare_for_new_get(getColumn, index);
      if (this.is_value_null(getColumn, index)) {
         return null;
      } else {
         byte[] result = this.getBytesInternal(getColumn, index);
         if (result != null && result.length != 0) {
            int type = this.get_value_type(getColumn, index);
            switch (type) {
               case 1:
               case 8:
               case 96:
                  String str = this.connection.conversion.CharBytesToString(result, result.length);
                  return Time.valueOf(str);
               case 12:
                  return this.connection.conversion.DateBytesToTime(result, result.length);
               default:
                  DBError.check_error(-4, "getTime");
                  return null;
            }
         } else {
            return null;
         }
      }
   }

   Timestamp getTimestampValue(boolean getColumn, int index) throws SQLException {
      this.connection.trace("Statement.getTimestamp");
      this.prepare_for_new_get(getColumn, index);
      if (this.is_value_null(getColumn, index)) {
         return null;
      } else {
         int type = this.get_value_type(getColumn, index);
         byte[] result = this.getBytesInternal(getColumn, index);
         if (result != null && result.length != 0) {
            switch (type) {
               case 1:
               case 8:
               case 96:
                  String str = this.connection.conversion.CharBytesToString(result, result.length);
                  return Timestamp.valueOf(str);
               case 12:
                  return this.connection.conversion.DateBytesToTimestamp(result, result.length);
               default:
                  DBError.check_error(-4, "getTimestamp");
                  return null;
            }
         } else {
            return null;
         }
      }
   }

   OracleInputStream getStreamInternal(boolean getColumn, int index) throws SQLException {
      --index;
      byte[] result = null;
      DBDataSet dbset;
      int row_index;
      if (getColumn) {
         dbset = this.defines;
         this.defines.getType(index);
         row_index = this.current_row;
      } else {
         dbset = this.binds_out;
         this.binds_out.getType(index);
         row_index = this.current_rank;
      }

      DBItem item = dbset.getData(index).getItem(row_index);
      return (OracleInputStream)item.stream_value;
   }

   InputStream getAsciiStreamValue(boolean getColumn, int index) throws SQLException {
      this.prepare_for_new_get(getColumn, index);
      if (this.is_value_null(getColumn, index)) {
         return null;
      } else {
         DBType dbtype = this.get_db_type(getColumn, index);
         DBConversion dbconv = this.connection.conversion;
         switch (dbtype.type) {
            case 1:
            case 8:
            case 96:
               if (dbtype.is_stream) {
                  OracleInputStream istream = this.getStreamInternal(getColumn, index);
                  if (istream.isClosed()) {
                     DBError.check_error(-27, "getAsciiStream");
                  }

                  return dbconv.ConvertStream(istream, 0);
               }

               byte[] result = this.getBytesInternal(getColumn, index);
               return dbconv.ConvertStream(new ByteArrayInputStream(result), 0);
            case 23:
            case 24:
               if (dbtype.is_stream) {
                  OracleInputStream istream = this.getStreamInternal(getColumn, index);
                  if (istream.isClosed()) {
                     DBError.check_error(-27, "getAsciiStream");
                  }

                  return dbconv.ConvertStream(istream, 2);
               }

               byte[] result = this.getBytesInternal(getColumn, index);
               return dbconv.ConvertStream(new ByteArrayInputStream(result), 2);
            default:
               DBError.check_error(-4, "getAsciiStream");
               return null;
         }
      }
   }

   InputStream getUnicodeStreamValue(boolean getColumn, int index) throws SQLException {
      this.prepare_for_new_get(getColumn, index);
      if (this.is_value_null(getColumn, index)) {
         return null;
      } else {
         DBType dbtype = this.get_db_type(getColumn, index);
         DBConversion dbconv = this.connection.conversion;
         switch (dbtype.type) {
            case 1:
            case 8:
            case 96:
               if (dbtype.is_stream) {
                  OracleInputStream istream = this.getStreamInternal(getColumn, index);
                  if (istream.isClosed()) {
                     DBError.check_error(-27, "getUicodeStream");
                  }

                  return dbconv.ConvertStream(istream, 1);
               }

               byte[] result = this.getBytesInternal(getColumn, index);
               return dbconv.ConvertStream(new ByteArrayInputStream(result), 1);
            case 23:
            case 24:
               if (dbtype.is_stream) {
                  OracleInputStream istream = this.getStreamInternal(getColumn, index);
                  if (istream.isClosed()) {
                     DBError.check_error(-27, "getUnicodeStream");
                  }

                  return dbconv.ConvertStream(istream, 3);
               }

               byte[] result = this.getBytesInternal(getColumn, index);
               return dbconv.ConvertStream(new ByteArrayInputStream(result), 3);
            default:
               DBError.check_error(-4, "getUnicodeStream");
               return null;
         }
      }
   }

   InputStream getBinaryStreamValue(boolean getColumn, int index) throws SQLException {
      this.prepare_for_new_get(getColumn, index);
      if (this.is_value_null(getColumn, index)) {
         return null;
      } else {
         DBType dbtype = this.get_db_type(getColumn, index);
         switch (dbtype.type) {
            case 1:
            case 8:
            case 23:
            case 24:
            case 96:
               if (dbtype.is_stream) {
                  OracleInputStream istream = this.getStreamInternal(getColumn, index);
                  if (istream.isClosed()) {
                     DBError.check_error(-27, "getBinaryStream");
                  }

                  return istream;
               }

               byte[] result = this.getBytesInternal(getColumn, index);
               return new ByteArrayInputStream(result);
            default:
               DBError.check_error(-4, "getBinaryStream");
               return null;
         }
      }
   }

   Object getObjectValue(boolean getColumn, int index) throws SQLException {
      this.connection.trace("Statement.getObject");
      this.prepare_for_new_get(getColumn, index);
      int column_type = this.get_value_type(getColumn, index);
      switch (column_type) {
         case 1:
         case 8:
         case 96:
            return this.getStringValue(getColumn, index);
         case 2:
         case 6:
            int scale = this.get_value_scale(getColumn, index);
            return this.getBigDecimalValue(getColumn, index, scale);
         case 11:
            return this.getRowidValue(getColumn, index);
         case 12:
            return this.getTimestampValue(getColumn, index);
         case 23:
         case 24:
            return this.getBytesValue(getColumn, index);
         case 102:
            return this.getCursorValue(getColumn, index);
         case 112:
            return this.getClobValue(getColumn, index);
         case 113:
            return this.getBlobValue(getColumn, index);
         case 114:
            return this.getBfileValue(getColumn, index);
         case 115:
            return this.getCfileValue(getColumn, index);
         default:
            DBError.check_error(-4, "getObjectValue");
            return null;
      }
   }

   int get_column_index(String columnName) throws SQLException {
      int column_num = this.dbdescription.length;

      for(int index = 0; index < column_num; ++index) {
         String str = this.dbdescription[index].getColumnName();
         if (str.equals(columnName)) {
            return index + 1;
         }
      }

      columnName = columnName.toUpperCase();

      for(int var6 = 0; var6 < column_num; ++var6) {
         String str = this.dbdescription[var6].getColumnName();
         if (str.equals(columnName)) {
            return var6 + 1;
         }
      }

      DBError.check_error(-6, "get_column_index");
      return 0;
   }

   int get_value_type(boolean getColumn, int index) {
      return getColumn ? this.defines.getType(index - 1).type : this.binds_out.getType(index - 1).type;
   }

   DBType get_db_type(boolean getColumn, int index) {
      return getColumn ? this.defines.getType(index - 1) : this.binds_out.getType(index - 1);
   }

   int get_value_scale(boolean getColumn, int index) {
      return 0;
   }

   boolean is_value_null(boolean getColumn, int index) throws SQLException {
      return getColumn ? this.defines.isNull(index - 1, this.current_row) : this.binds_out.isNull(index - 1, this.current_rank);
   }

   int get_internal_type(int external_type) throws SQLException {
      switch (external_type) {
         case -14:
            return 115;
         case -13:
            return 114;
         case -12:
            return 112;
         case -11:
            return 113;
         case -10:
            return 102;
         case -8:
            return 11;
         case -7:
         case -6:
         case -5:
         case 2:
         case 3:
         case 4:
         case 5:
         case 6:
         case 7:
         case 8:
            return 6;
         case -4:
            return 24;
         case -3:
         case -2:
            return 23;
         case -1:
            return 8;
         case 1:
            return 96;
         case 12:
            return 1;
         case 91:
         case 92:
         case 93:
            return 12;
         default:
            DBError.check_error(-4, "get_internal_type");
            return 0;
      }
   }

   public int get_internal_size(int type) {
      return 0;
   }

   synchronized void describe() throws SQLException {
      if (this.dbdescription == null) {
         try {
            this.connection.needLine();
            this.dbdescription = this.connection.db_access.describe(this.dbstmt);
         } catch (IOException e) {
            DBError.check_error(e);
         }
      }

   }

   public void finalize() throws SQLException {
      this.connection.trace("Statement.finalize");
      this.close();
   }

   void freeLine() throws SQLException {
      if (this.streams != null) {
         try {
            this.streams.close();
         } catch (IOException e) {
            DBError.check_error(e);
         }
      }

   }
}
