package oracle.sqlnet;

import java.io.IOException;
import java.net.Socket;

public class SQLnet implements SQLnetDef {
   private String conDataOut = new String();
   private String conDataIn = new String();
   private int lastError;
   private boolean connected = false;
   private int sduSize;
   private int tduSize;
   private Socket socket;
   private byte[] inPacket = new byte[4086];
   private SQLnetOutputStream netOut;
   private SQLnetInputStream netIn;
   private SQLnetBufferedOutputStream netBufOut;
   private SQLnetBufferedInputStream netBufIn;
   private static final int SQLnetErrorNoSocket = 1;
   private static final int SQLnetErrorRefused = 2;
   private static final int SQLnetErrorUnexpectedPacket = 3;
   private static final int SQLnetErrorAlreadyConnected = 4;
   private static final int SQLnetErrorInvalidCondata = 5;

   public void Connect(String TNSdesc) throws IOException {
      int sduSize = 0;
      int tduSize = 0;
      if (TNSdesc.indexOf(40) != -1) {
         NVFactory nvf = new NVFactory();
         NVNavigator nvn = new NVNavigator();
         NVPair addr = nvn.findNVPairRecurse(nvf.createNVPair(TNSdesc), "address");
         NVPair cnvp = nvn.findNVPair(addr, "ProToCol");
         if (cnvp == null) {
            throw new IOException("Protocol not specified.");
         }

         String prot = cnvp.getAtom();
         if (!prot.equals("TCP") && !prot.equals("tcp")) {
            this.lastError = 5;
            throw new IOException("Invalid Connect Data");
         }

         cnvp = nvn.findNVPair(addr, "Host");
         if (cnvp == null) {
            throw new IOException("Hostname not specified.");
         }

         String host = cnvp.getAtom();
         cnvp = nvn.findNVPair(addr, "Port");
         if (cnvp == null) {
            throw new IOException("Port not specified.");
         }

         int port = Integer.parseInt(cnvp.getAtom());
         cnvp = nvn.findNVPair(addr, "sduSize");
         if (cnvp != null) {
            sduSize = Integer.parseInt(cnvp.getAtom());
         }

         cnvp = nvn.findNVPair(addr, "tduSize");
         if (cnvp != null) {
            tduSize = Integer.parseInt(cnvp.getAtom());
         }

         this.Connect(host, port, sduSize, tduSize, TNSdesc);
      } else if (TNSdesc.indexOf(58) != -1) {
         int firstColon = TNSdesc.indexOf(58);
         int secondColon = TNSdesc.lastIndexOf(58);
         if (firstColon == -1 || secondColon == -1) {
            throw new IOException("Connect string parsing error.");
         }

         String host = TNSdesc.substring(0, firstColon);
         int port = Integer.parseInt(TNSdesc.substring(firstColon + 1, secondColon));
         String SID = TNSdesc.substring(secondColon + 1, TNSdesc.length());
         this.Connect(host, port, SID);
      }

   }

   public void Connect(String host, int port) throws IOException {
      this.Connect(host, port, 0, 0, this.conDataOut);
   }

   public void Connect(String host, int port, String SID) throws IOException {
      String conData = "(DESCRIPTION=(CONNECT_DATA=(SID=" + SID + ")(CID=(PROGRAM=)(HOST=)(USER=oracle)))" + "(ADDRESS_LIST=(ADDRESS=(COMMUNITY=TCP.amd.com)" + "(PROTOCOL=tcp)(HOST=" + host + ")(PORT=" + port + "))))";
      this.Connect(host, port, 0, 0, conData);
   }

   public synchronized void Connect(String host, int port, int sduSize, int tduSize, String conData) throws IOException {
      this.lastError = 0;
      if (this.connected) {
         this.lastError = 4;
         throw new IOException("Already connected");
      } else {
         this.sduSize = sduSize <= 4086 && sduSize >= 512 ? sduSize : 2048;
         this.tduSize = tduSize <= 32767 && tduSize >= 255 ? tduSize : 32767;
         this.socket = new Socket(host, port);
         if (this.socket == null) {
            this.lastError = 1;
            throw new IOException("No socket");
         } else {
            this.netIn = new SQLnetInputStream(this.socket.getInputStream());
            this.netOut = new SQLnetOutputStream(this.socket.getOutputStream());

            while(!this.connected && this.lastError == 0) {
               this.netOut.writeConnectPacket(this.sduSize, this.tduSize, conData);
               this.netIn.readPacket(this.inPacket);
               SQLnetPacket pak = new SQLnetPacket(this.inPacket);
               switch (pak.getType()) {
                  case 2:
                     this.sduSize = pak.getSduSize();
                     this.tduSize = pak.getTduSize();
                     if (this.tduSize < this.sduSize) {
                        this.sduSize = this.tduSize;
                     }

                     this.conDataIn = pak.getDataString(this.netIn);
                     this.netBufOut = new SQLnetBufferedOutputStream(this.netOut, this.sduSize);
                     this.netBufIn = new SQLnetBufferedInputStream(this.netIn, this.netBufOut, this.sduSize);
                     this.connected = true;
                     break;
                  case 4:
                     this.conDataIn = pak.getDataString(this.netIn);
                     this.socket.close();
                     this.lastError = 2;
                     throw new IOException("Refused:" + this.conDataIn);
                  case 5:
                     this.connected = false;
                     this.conDataIn = pak.getDataString(this.netIn);
                     this.socket.close();
                     this.Connect(this.conDataIn);
                  case 11:
                     break;
                  default:
                     this.socket.close();
                     this.lastError = 3;
                     throw new IOException("Unexpected Packet. Type = " + pak.getType());
               }
            }

         }
      }
   }

   public synchronized void Disconnect() throws IOException {
      if (this.connected) {
         this.netBufOut.close();
         this.socket.close();
         this.connected = false;
      }

   }

   public void Break() throws IOException {
      this.netOut.writeMarkerPacket();
   }

   public void reset() throws IOException {
      this.netOut.writeMarkerPacket((byte)2);
      if (!this.netIn.gotResetPacket()) {
         this.netIn.readPacket(this.inPacket);
         SQLnetPacket pak = new SQLnetPacket(this.inPacket);

         while(pak.getType() != 12 || pak.markerData() != 2) {
            this.netIn.readPacket(this.inPacket);
            pak.setPacket(this.inPacket);
         }
      }

      this.netIn.resetComplete();
   }

   public SQLnetBufferedInputStream GetBufferedInputStream() {
      return this.netBufIn;
   }

   public SQLnetBufferedOutputStream GetBufferedOutputStream() {
      return this.netBufOut;
   }

   public String GetConnectData() {
      return new String(this.conDataIn);
   }

   public void SetConnectData(String conData) {
      this.conDataOut = new String(conData);
   }

   public int GetLastError() {
      int lastError = this.lastError;
      this.lastError = 0;
      return lastError;
   }
}
